==> dasu_4.11.jar: edu/northwestern/dasu/measurement/ame/AMEManager.java <==
/**
 * Dasu Project
 *
 * File:         AMEManager.java
 * RCS:          $Id: AMEManager.java,v 1.0 Oct 8, 2012 8:17:24 AM mas939 Exp $
 * Description:  AMEManager class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 8, 2012 at 8:17:24 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.ame;

import java.io.File;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NoSuchObjectException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import edu.northwestern.cs.aqualab.Cattle.json.JSONObject;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.dasu.coordination.DistributedCoordinator;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.dynamic.coordination.UnlimitedDistributedCoordinator;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.sandbox.AntJVM;
import edu.northwestern.dasu.sandbox.applet.RmiAMCoordinatorImpl;
import edu.northwestern.dasu.sandbox.applet.RmiAMEServerInterface;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AMEManager class ...
 */
public class AMEManager extends UnicastRemoteObject implements RmiAMEServerInterface {

	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");

	public static Hashtable<SourceType, AMEngine> AMEngines = new Hashtable<SourceType, AMEngine>();
	private static AntJVM antJvm = null;
	public static RmiAMCoordinatorImpl rmiAMCoordinator = null;	
	private static AMEManager self;
	private static Boolean isActive = false;
	private static Object halt = new Object();

	/**
	 * @throws RemoteException
	 */
	protected AMEManager() throws RemoteException {
		super();
	}
	
	public boolean isActive() {
		return isActive;
	}
	

	public static AMEManager getInstance() {
		
		if (self == null) {
			try {
				self = new AMEManager();
			}
			catch (Exception e) {
				LOGGER.info("AME: ERROR: " + e.getLocalizedMessage());
			}
		}
		return self;
	}
 
	public void initializeJVM() {
		LOGGER.info("AME: Starting AntJVM");
		antJvm = new AntJVM(DasuConfiguration.getInstance().getMaxAMExperiments(),
				DasuConfiguration.getInstance().getMaxMemJVM());
		antJvm.start();
	}
	
	public boolean isJvmActive() {
		
		if ( antJvm != null) {
			return antJvm.isAlive();
		}
		
		return false;
	}
	
	//up to three concurrent experiments can run concurrently
	private void startDistributedCoordinator(String filename, String sigFilename, Integer taskId, SourceType engineId) {
		
		try {
			
			DistributedCoordinator coordinator = null;
	
			ClassLoader cl = DasuManager.getDasuClassLoader();
			Properties map = DasuConfiguration.getInstance().getProbeModulesProperties();
			if ( map.containsKey("distributedCoordinator") )
				coordinator = (DistributedCoordinator) cl.loadClass(map.getProperty("distributedCoordinator"))
					.getDeclaredConstructor(new Class[]{SourceType.class, String.class}).newInstance(engineId, "DasuDistributedCoordinator_");
			else
				coordinator = new UnlimitedDistributedCoordinator(engineId, "DasuDistributedCoordinator_");
			
			AMEngines.put(engineId, new AMEngine(filename, sigFilename, taskId, engineId, coordinator));
			
			LOGGER.info("AME:  starting coordinator version " + coordinator.getVersion() + " of type " 
					+ coordinator.getClass().getSimpleName());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		
		
	}
	
	public boolean initializeRmiServers()  {
		
		try {
			
			//convert to uri to avoid bug in the codebase parser that uses space separated url strings
			File f = new File(this.getClass().getProtectionDomain().getCodeSource().getLocation().getFile());
			String dir = f.toURI().toASCIIString().replace("file:", "file:///");
			System.getProperties().put("java.rmi.server.useCodebaseOnly", "true");
			System.getProperties().put("java.rmi.server.codebase", dir);
			System.out.println(System.getProperties().get("java.rmi.server.codebase"));
			System.getProperties().put("java.rmi.server.hostname", "127.0.0.1");
			
			try {
				LocateRegistry.createRegistry(1099);
				LOGGER.info("RMI: java RMI registry created.");
			}
			catch (Exception e) {
				LOGGER.info("RMI: " + e.getMessage());
			}			
	    	
			rmiAMCoordinator = new RmiAMCoordinatorImpl();
	    	Naming.rebind("rmi://localhost/RmiAMCCoordinator", rmiAMCoordinator);
	    	LOGGER.info("RMI: RmiAMCCoordinator bound in registry");
	    	
	    	Naming.rebind("rmi://localhost/RmiAMEServer", this);
	    	LOGGER.info("RMI: RmiAMEServer bound in registry");	    	
	    	
		}
		catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}
	
	public void initialize() throws Exception {

		if (rmiAMCoordinator == null) {
			if (!initializeRmiServers()) {
				LOGGER.severe("RMI: ERROR: Could not initialized RMI Servers!");
				return;
			}
		}

		if ( !isJvmActive() ) {
			//initialize external JVM
			initializeJVM();
		}
		isActive = true;
	}
	
	private void stopRmiServers() {
		
		try {
			
			LOGGER.info("STOP: Stopping RmiServers");
			
			Naming.unbind("rmi://localhost/RmiAMCCoordinator");
			UnicastRemoteObject.unexportObject(rmiAMCoordinator, true);
			Naming.unbind("rmi://localhost/RmiAMEServer");
			UnicastRemoteObject.unexportObject(this, true);

		} catch (NoSuchObjectException e) {			
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (NotBoundException e) {
			e.printStackTrace();
		}

	}
	
	public void stop() {
		
		LOGGER.info("STOP: Stopping AMEManager...");
		synchronized(halt) {
			isActive = false;
			halt.notifyAll();
		}
		
		if ( antJvm != null )
			antJvm.stopJVM();
		if ( rmiAMCoordinator!= null)
			stopRmiServers();
		
		antJvm = null;
		rmiAMCoordinator = null;
		stopAMEngines();
	}
	
	public void stopAMEngines() {
		
		for ( SourceType st : AMEngines.keySet() ) {
			AMEngines.get(st).stop();
		}
	}
		
	/* RmiAMEServerInterface */
	
	public ArrayList<String> getAMExperiment(String engineId) throws RemoteException {
		
		LOGGER.info("AME: experiment request " + engineId);
		int AMECheckupInterval = 60; //wait for one minute
		
		if (Main.AME_DEBUG_SKIP_DOWNLOAD)
			AMECheckupInterval = 15;

		synchronized(halt) {
			try {
				halt.wait(AMECheckupInterval*1000);						
			} catch (InterruptedException e) {
				isActive = false;
				LOGGER.info("STOP: abort request " + engineId);
			}
		}
		
		if ( !isActive )
			return null;

		//testing!
//		try {
//			if ( engineId.equals("AME") )
//				Thread.sleep(5*1000);
//			if ( engineId.equals("AME2") )
//				Thread.sleep(10*1000);
//			if ( engineId.equals("AME3") )
//				Thread.sleep(15*1000);
//			ArrayList<String> ret = new ArrayList();
//			ret.add("/Users/mas939/Mario's Stuff/NORTHWESTERN/Dropbox/workspace/ISP Interference/testaz/testdasu/plugins/dasu/AMExperiment.drl");
//			ret.add("/Users/mas939/Mario's Stuff/NORTHWESTERN/Dropbox/workspace/ISP Interference/testaz/testdasu/plugins/dasu/AMExperiment.signature");
//			ret.add("8000");
//			ret.add(engineId);
//			return ret;
//
//		} catch (InterruptedException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
		
		if (Main.AME_DEBUG_SKIP_DOWNLOAD) {
			Integer taskId = 8980322;
			ArrayList<String> ret = new ArrayList<String>();
			ret.add(Main.getDir() + "/AMExperiment." + engineId + ".drl");
			ret.add(Main.getDir() + "/AMExperiment." + engineId + ".signature");
			LOGGER.info("AME: DasuManager.class: Skipping AMExperiment.drl download due to DEBUG flag!");
			ret.add(taskId.toString());
			ret.add(engineId);
			startDistributedCoordinator(ret.get(0), ret.get(1), taskId, SourceType.valueOf(engineId));
			return ret;
		}
		
		String myIp = Main.getPublicIpAddress();
		
		String resource = DasuConfiguration.DASU_SCRIPT_AME + "?ip=" + myIp +
			"&UUID=" + DasuConfiguration.getInstance().getUUID();
		
		if ((NetworkAdminASNLookupImpl.getInstance() != null) 
				&& (NetworkAdminASNLookupImpl.getInstance().getASNInfo() != null)) {

			resource = resource.concat("&asn=" + NetworkAdminASNLookupImpl.getInstance().getASNInfo().getAS() +
				"&cc=" + NetworkAdminASNLookupImpl.getInstance().getASNInfo().getCountry());
		}
		
		/* load file from server */							
		JSONObject json = DasuManager.getInstance().getConfigService().GETData(DasuConfiguration.DASU_SCRIPT_AME + 
				DasuConfiguration.getInstance().getResourceScriptRequest());
		
		if (json == null) {
			LOGGER.info("AME: DasuManager.class: AMEngine: No new experiment to load...");
			return null;
		}

		String interfFile = DasuConfiguration.DASU_AME_FILENAME_PREFIX + "." + engineId + DasuConfiguration.DASU_AME_FILENAME_SUFFIX;
		String interfSignature = DasuConfiguration.DASU_AME_FILENAME_PREFIX + "." + engineId + DasuConfiguration.DASU_AME_SIGNATURE_SUFFIX;
		String dir = Main.getDir() + java.io.File.separator;
		String configUrl = null;
		Integer taskId = null;
		String signatureUrl = null;
		boolean isNewFile = false;
		boolean isSigFile = false;

		try {
			
			Iterator<String> iter = json.keys();
			while ( iter.hasNext() ) {
				String key = iter.next();
				if (key.equals("dasu.ame.drl")) 
					configUrl = json.getString(key);
				if (key.equals("dasu.ame.signature"))
					signatureUrl = json.getString(key);
				if (key.equals("dasu.ame.taskid"))
					taskId = json.getInt(key);
				
			}
		}
		catch (Exception e) {
			//e.printStackTrace();
			LOGGER.warning("AME: ERROR: invalid json object");
		}
		
		// taskId smaller than 0 are not valid active task ids
		if (taskId != null && taskId > 0 ) {
				//&& !getPersistentDataManager().getTaskLogHistory().containsKey(taskId)) {
			
			LOGGER.info("AME: DasuManager.class: New AM experiment found... downloading");

			isNewFile = DasuManager.getInstance().getConfigService().GETFile(configUrl, dir+interfFile);
			isSigFile = DasuManager.getInstance().getConfigService().GETFile(signatureUrl, dir+interfSignature);
			
			if ((isNewFile) && (isSigFile)) {
				
				ArrayList<String> ret = new ArrayList<String>();
				ret.add(dir + interfFile);
				ret.add(dir + interfSignature);
				ret.add(taskId.toString());
				ret.add(engineId);
				startDistributedCoordinator(ret.get(0), ret.get(1), taskId, SourceType.valueOf(engineId));
				return ret;				
				
			}
			else {
				LOGGER.info("AME: DasuManager.class: Active Measurement file and/or signature file missing!");
			}
		} else {
			
			LOGGER.info("AME: Invalid taskId");
		}

		return null;
	}

	public void AMECompleted(String engineId) {
		
		
		if ( AMEngines.containsKey(SourceType.valueOf(engineId)) ) {
			AMEngine ame = AMEngines.remove(SourceType.valueOf(engineId));
			Integer taskId = ame.taskId;
			ame.stop();
			
			DasuManager.getPersistentDataManager().getTaskLogHistory().put(taskId, Util.currentGMTTime());
			ReportableFact rf = new ReportableFact("completedTask");
			rf.getContentsMap().put("GMTTime", Util.currentGMTTime());
			rf.getContentsMap().put("taskId", taskId);
			Statistics.getInstance().addReportableFact(rf);
			
			LOGGER.info("AME: completed task from on engine " + engineId + " with taskId " + taskId);

		}
		else
			LOGGER.severe("AME: ERROR: Removing engineId without Distributed Coordinator!");
	}
	
	public boolean isServerAlive() {		
		return isJvmActive();
	}
	

}

==> dasu_5.1.jar: edu/northwestern/dasu/measurement/ame/AMEManager.java <==
/**
 * Dasu Project
 *
 * File:         AMEManager.java
 * RCS:          $Id: AMEManager.java,v 1.0 Oct 8, 2012 8:17:24 AM mas939 Exp $
 * Description:  AMEManager class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 8, 2012 at 8:17:24 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2012, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement.ame;

import java.io.File;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NoSuchObjectException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.coordination.DistributedCoordinator;
import edu.northwestern.dasu.drools.ReportableFact;
import edu.northwestern.dasu.dynamic.coordination.UnlimitedDistributedCoordinator;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.types.SourceType;
import edu.northwestern.dasu.sandbox.AntJVM;
import edu.northwestern.dasu.sandbox.applet.RmiAMCoordinatorImpl;
import edu.northwestern.dasu.sandbox.applet.RmiAMEServerInterface;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AMEManager class ...
 */
public class AMEManager extends UnicastRemoteObject implements RmiAMEServerInterface {

	private static final long serialVersionUID = 1L;

	private final static java.util.logging.Logger LOGGER = java.util.logging.Logger.getLogger("edu.northwestern.dasu");

	public static Hashtable<SourceType, AMEngine> AMEngines = new Hashtable<SourceType, AMEngine>();
	private static AntJVM antJvm = null;
	public static RmiAMCoordinatorImpl rmiAMCoordinator = null;	
	private static AMEManager self;
	private static Boolean isActive = false;
	private static Object halt = new Object();

	/**
	 * @throws RemoteException
	 */
	protected AMEManager() throws RemoteException {
		super();
	}
	
	public boolean isActive() {
		return isActive;
	}
	
	public static AMEManager getInstance() {
		
		if (self == null) {
			try {
				self = new AMEManager();
			}
			catch (Exception e) {
				LOGGER.info("AME: ERROR: " + e.getLocalizedMessage());
			}
		}
		return self;
	}
 
	private void initializeJVM() {
		LOGGER.info("AME: Starting AntJVM");
		antJvm = new AntJVM(DasuConfiguration.getInstance().getMaxAMExperiments(),
				DasuConfiguration.getInstance().getMaxMemJVM());
		antJvm.start();
	}
	
	public boolean isJvmActive() {
		
		if ( antJvm != null) {
			return antJvm.isAlive();
		}
		
		return false;
	}
	
	//up to three concurrent experiments can run concurrently
	private void startDistributedCoordinator(String filename, String sigFilename, Integer taskId, SourceType engineId,
			JSONObject distributedCoordinatorParams) {
		
		try {
			
			DistributedCoordinator coordinator = null;
	
			ClassLoader cl = DasuManager.getDasuClassLoader();
			Properties map = DasuConfiguration.getInstance().getProbeModulesProperties();
			if ( map.containsKey("distributedCoordinator") )
				
				//throws exception if coordinator can't be initialized
				coordinator = (DistributedCoordinator) cl.loadClass(map.getProperty("distributedCoordinator"))
					.getDeclaredConstructor(new Class[]{SourceType.class, String.class, JSONObject.class}).newInstance(engineId, "DasuDistributedCoordinator_", distributedCoordinatorParams);
			else
				coordinator = new UnlimitedDistributedCoordinator(engineId, "DasuDistributedCoordinator_", distributedCoordinatorParams);
			
			AMEngines.put(engineId, new AMEngine(filename, sigFilename, taskId, engineId, coordinator));
			
			LOGGER.info("AME:  starting coordinator version " + coordinator.getVersion() + " of type " 
					+ coordinator.getClass().getSimpleName());
		}
		catch (Exception e) {
			LOGGER.warning("AME: " + e.getMessage());
			
			ReportableFact rf = new ReportableFact("ErrorEvent");
			rf.addContentsObject("taskId", taskId);
			rf.addContentsObject("report", "Could not start AMEngine for experiment");
			rf.addContentsObject("reason", e.getMessage());
			Statistics.getInstance().addReportableFact(rf);

		}
		
		
	}
	
	public boolean initializeRmiServers()  {
		
		try {
			
			//convert to uri to avoid bug in the codebase parser that uses space separated url strings
			File f = new File(DasuManager.getInstance().getClass().getProtectionDomain().getCodeSource().getLocation().getFile());			
			String dir = f.toURI().toASCIIString().replace("file:", "file:///");
		    File ex = new File(DasuManager.updater.getJarPath(DasuConfiguration.getInstance().getInferenceVersion()));
		    String externalJar = ex.toURI().toASCIIString().replace("file:", "file:///");
		    
			System.getProperties().put("java.rmi.server.useCodebaseOnly", "true");
			System.getProperties().put("java.rmi.server.codebase", externalJar + " " + dir);			
			System.getProperties().put("java.rmi.server.hostname", "localhost");
			
			try {
				LocateRegistry.createRegistry(1099);
				LOGGER.info("RMI: java RMI registry created.");
			}
			catch (Exception e) {
				LOGGER.info("RMI: " + e.getMessage());
			}			
	    	
			rmiAMCoordinator = new RmiAMCoordinatorImpl();
	    	Naming.rebind("rmi://localhost/RmiAMCCoordinator", rmiAMCoordinator);
	    	LOGGER.info("RMI: RmiAMCCoordinator bound in registry");
	    	
	    	Naming.rebind("rmi://localhost/RmiAMEServer", this);
	    	LOGGER.info("RMI: RmiAMEServer bound in registry");
	    	
		}
		catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}
	
	public void initialize() throws Exception {

		if (rmiAMCoordinator == null) {
			if (!initializeRmiServers()) {
				LOGGER.severe("RMI: ERROR: Could not initialized RMI Servers!");
				return;
			}
		}

		if ( !isJvmActive() ) {
			//initialize external JVM
			initializeJVM();
		}
		isActive = true;
	}
	
	private void stopRmiServers() {
		
		try {
			
			LOGGER.info("STOP: Stopping RmiServers");
			
			Naming.unbind("rmi://localhost/RmiAMCCoordinator");
			UnicastRemoteObject.unexportObject(rmiAMCoordinator, true);
			Naming.unbind("rmi://localhost/RmiAMEServer");
			UnicastRemoteObject.unexportObject(this, true);

		} catch (NoSuchObjectException e) {			
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (NotBoundException e) {
			e.printStackTrace();
		}

	}
	
	public void stop() {
		
		LOGGER.info("STOP: Stopping AMEManager...");
		synchronized(halt) {
			isActive = false;
			halt.notifyAll();
		}
		
		if ( antJvm != null )
			antJvm.stopJVM();
		if ( rmiAMCoordinator!= null)
			stopRmiServers();
		
		antJvm = null;
		rmiAMCoordinator = null;
		stopAMEngines();
	}
	
	public void stopAMEngines() {
		
		for ( SourceType st : AMEngines.keySet() ) {
			AMEngines.get(st).stop();
		}
	}
		
	/* RmiAMEServerInterface */
	
	public ArrayList<String> getAMExperiment(String engineId) throws RemoteException {
		
		LOGGER.info("AME: experiment request " + engineId);
		int AMECheckupInterval = 60; //wait for one minute
		
		if (Main.AME_DEBUG_SKIP_DOWNLOAD)
			AMECheckupInterval = 15;

		synchronized(halt) {
			try {
				halt.wait(AMECheckupInterval*1000);						
			} catch (InterruptedException e) {
				isActive = false;
				LOGGER.info("STOP: abort request " + engineId);
			}
		}
		
		if ( !isActive )
			return null;
		
		if (Main.AME_DEBUG_SKIP_DOWNLOAD) {
			
			
			JSONObject json = DasuManager.getInstance().getConfigService().GETData(DasuConfiguration.DASU_SCRIPT_AME + 
					DasuConfiguration.getInstance().getResourceScriptRequest());
			JSONObject distributedCoordinatorParams = (JSONObject) json.get("dasu.ame.distributedcoordinator.params");
			
			Integer taskId = 9980322;
			ArrayList<String> ret = new ArrayList<String>();
			ret.add(Main.getDir() + "/AMExperiment." + engineId + ".drl");
			ret.add(Main.getDir() + "/AMExperiment." + engineId + ".signature");
			LOGGER.info("AME: DasuManager.class: Skipping AMExperiment.drl download due to DEBUG flag!");
			ret.add(taskId.toString());
			ret.add(engineId);
			startDistributedCoordinator(ret.get(0), ret.get(1), taskId, SourceType.valueOf(engineId), distributedCoordinatorParams);
			return ret;
		}
				
		/* load file from server */							
		JSONObject json = DasuManager.getInstance().getConfigService().GETData(DasuConfiguration.DASU_SCRIPT_AME + 
				DasuConfiguration.getInstance().getResourceScriptRequest());
		
		if (json == null) {
			LOGGER.info("AME: DasuManager.class: AMEngine: No new experiment to load...");
			return null;
		}

		String interfFile = DasuConfiguration.DASU_AME_FILENAME_PREFIX + "." + engineId + DasuConfiguration.DASU_AME_FILENAME_SUFFIX;
		String interfSignature = DasuConfiguration.DASU_AME_FILENAME_PREFIX + "." + engineId + DasuConfiguration.DASU_AME_SIGNATURE_SUFFIX;
		String dir = Main.getDir() + java.io.File.separator;
		String configUrl = null;
		Long taskId = null;
		String signatureUrl = null;
		boolean isNewFile = false;
		boolean isSigFile = false;
		JSONObject distributedCoordinatorParams = null;

		try {
			
			Iterator<String> iter = json.keySet().iterator();
			while ( iter.hasNext() ) {
				String key = iter.next();
				if (key.equals("dasu.ame.drl")) 
					configUrl = json.get(key).toString();
				else if (key.equals("dasu.ame.signature"))
					signatureUrl = json.get(key).toString();
				else if (key.equals("dasu.ame.taskid"))
					taskId = (Long) json.get(key);
				else if ( key.equals("dasu.ame.distributedcoordinator.params"))
					distributedCoordinatorParams = (JSONObject)json.get(key);
				
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			LOGGER.warning("AME: ERROR: invalid json object");
		}
		
		// taskId smaller than 0 are not valid active task ids
		if (taskId != null && taskId > 0 ) {
				//&& !getPersistentDataManager().getTaskLogHistory().containsKey(taskId)) {
			
			LOGGER.info("AME: New AM experiment found... downloading");

			isNewFile = DasuManager.getInstance().getConfigService().GETFile(configUrl, dir+interfFile);
			isSigFile = DasuManager.getInstance().getConfigService().GETFile(signatureUrl, dir+interfSignature);
			
			if ((isNewFile) && (isSigFile)) {
				
				ArrayList<String> ret = new ArrayList<String>();
				ret.add(dir + interfFile);
				ret.add(dir + interfSignature);
				ret.add(taskId.toString());
				ret.add(engineId);
				startDistributedCoordinator(ret.get(0), ret.get(1), taskId.intValue(), SourceType.valueOf(engineId), distributedCoordinatorParams);
				return ret;				
				
			}
			else {
				LOGGER.info("AME: DasuManager.class: Active Measurement file and/or signature file missing!");
			}
		} else {
			
			LOGGER.info("AME: Invalid taskId");
		}

		return null;
	}

	public void AMECompleted(String engineId, boolean success) {
		
		if ( AMEngines.containsKey(SourceType.valueOf(engineId)) ) {
			AMEngine ame = AMEngines.remove(SourceType.valueOf(engineId));
			Long statrTime = ame.startTime;
			Integer taskId = ame.taskId;
			ame.stop();
			
			AMCoordinatorV2.AMECompleted(SourceType.valueOf(engineId));
			
			if (success)
				LOGGER.info("AME: completed task from on engine " + engineId + " with taskId " + taskId);
			else
				LOGGER.info("AME: unsuccessful task from on engine " + engineId + " with taskId " + taskId);
			
			DasuManager.getPersistentDataManager().getTaskLogHistory().put(taskId, Util.currentGMTTime());
			ReportableFact rf = new ReportableFact("CompletedTask");
			rf.addContentsObject("success", success);
			rf.addContentsObject("taskId", taskId);
			rf.addContentsObject("taskLaunchTime", statrTime);
			rf.addContentsObject("taskEndTime", System.currentTimeMillis());
			rf.addContentsObject("maxBWUnitConsumptionScheduler", DasuConfiguration.getInstance().getMaxBWUnitConsumptionScheduler());
			rf.addContentsObject("maxBWUtilizationAllowedScheduler", DasuConfiguration.getInstance().getMaxBWUtilizationAllowedScheduler());
			rf.addContentsObject("maxCpuUtilizationAllowedScheduler", DasuConfiguration.getInstance().getMaxCpuUtilizationAllowedScheduler());
			Statistics.getInstance().addReportableFact(rf);
		}
		else
			LOGGER.severe("AME: ERROR: Removing engineId without Distributed Coordinator!");
	}
	
	public boolean isServerAlive() {		
		return isJvmActive();
	}

	public void commitReportableFact(ReportableFact fpr) throws RemoteException {
		LOGGER.info("RMI: " + fpr);
		AMCoordinatorV2.commitReportableFact(fpr);		
	}
	

}
